/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

package com.monblocnotes.review;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
//import java.net.URL;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class HelpDialog extends Dialog {
	
	private static final int BUTTON_HEIGHT = 30;
	private static final int BUTTON_LENGTH = 100;
	private static final int BROWSER_HEIGHT = 400;
	private static final int BROWSER_LENGTH = 600;
	
	private static final String CONTENTS_FILENAME = "helpContents.html";
	private static final String ABOUT_FILENAME = "helpAbout.html";

	private Shell shell = null;
	private Shell parent = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	
	private static final String VERSION_NUMBER = "V0.5";
	

	public HelpDialog(Shell parent, int style) {
		super(parent);	
	}
	
	public HelpDialog(Shell parent) {
		super(parent);
	}
	
	// Returns false if browser could not be created.
	// TODO display an error message if browser can't be created, instead of doing nothing.
	private boolean prepareHelpDialog(String helpFileName) {
		
		parent = getParent(); 
		shell = new Shell(parent, SWT.SHELL_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(resReview.getString("HW_title") + " - " + VERSION_NUMBER);
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		
		// Apply a one-column grid layout to the shell.
		GridLayout contentsGL = new GridLayout(1, false);
		shell.setLayout(contentsGL);

		// Create a browser.
		final Browser contentsBrw;
		try {
			contentsBrw = new Browser(shell, SWT.NONE);
		} catch (SWTError e) {
			System.out.println("Could not instantiate Browser: " + e.getMessage());
			shell.dispose();
			return false;
		}
		// Set size.
		contentsBrw.setBounds(0,0,BROWSER_LENGTH, BROWSER_HEIGHT);
		GridData contentsGD = new GridData(BROWSER_LENGTH, BROWSER_HEIGHT);
		contentsGD.horizontalAlignment = GridData.FILL;
		contentsGD.verticalAlignment = GridData.FILL;
		contentsGD.grabExcessHorizontalSpace = true;
		contentsGD.grabExcessVerticalSpace = true;
		contentsBrw.setLayoutData(contentsGD);		
		
		// Get access to root help file.
		ClassLoader cl = this.getClass().getClassLoader();
		/*
		URL url = cl.getResource(helpFileName);
		if (url == null) {
			System.out.println("Could not instantiate URL for " + helpFileName);
			shell.dispose();
			return false;
		}
		*/
		/*
		 * In a first version, we were directly opening the help file using an URL pointing
		 * to it (the URL built in the commented code above). This was working OK on
		 * Linux. But was failing on Windows...
		 * 
		 */
		// Get contents of help file. Its contents is encoded in UTF-8.
		String localEncoding = System.getProperty ("file.encoding");
		System.out.println("Local encoding: " + localEncoding);
		
		InputStream is = cl.getResourceAsStream(helpFileName);
		InputStreamReader isr;
		try {
			isr = new InputStreamReader(is, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			System.out.println("Could not set local encoding: " + e.getMessage());
			shell.dispose();
			return false;
		}
		BufferedReader br = new BufferedReader(isr);
		StringBuffer sb = new StringBuffer();
		String line;
		try {
			while ((line = br.readLine()) != null) {
				sb.append(line);
			}
			br.close();
			isr.close();
			is.close();
		} catch (IOException e) {
			System.out.println("Could not read contents of help file: " +
					           e.getMessage());
			shell.dispose();
			return false;
		}

		// Request to display it, using local encoding.
		//System.out.println("Displaying contents of " + url.toString());
		byte [] localEncodingBytes = sb.toString().getBytes();
		try {
			String localEncodingStr = new String(localEncodingBytes, localEncoding);
			contentsBrw.setText(localEncodingStr);
		} catch (UnsupportedEncodingException e) {
			System.out.println("Could not convert encoding: " + e.getMessage());
			shell.dispose();
			return false;
		}
		
		// Prepare grid data, that will be used to center exit button, and to give it required size.
		GridData buttonGD = new GridData(BUTTON_LENGTH, BUTTON_HEIGHT);
		buttonGD.horizontalAlignment = SWT.CENTER;

		// Create exit button.
		Button exitBtn = new Button(shell, SWT.PUSH);
		exitBtn.setText(resReview.getString("HW_exit"));
		// Center it.
		//exitBtn.setLayoutData(buttonGD);
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}
		});
		
		return true;

	}
	
	public boolean prepareContentsDialog() {
		return prepareHelpDialog(CONTENTS_FILENAME);
	}
	
	public boolean prepareAboutDialog() {
		return prepareHelpDialog(ABOUT_FILENAME);
	}
	
	public void open() {
		
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}

	}

}
